@@ -15,17 +15,125 @@
 #include "retrofit_utils.hpp"
 
 // [[Rcpp::export]]
-Rcpp::List new_NoiseKriging(arma::vec y,
-                            arma::vec noise,
-                            arma::mat X,
-                            std::string kernel,
-                            std::string regmodel = "constant",
-                            bool normalize = false,
-                            std::string optim = "BFGS",
-                            std::string objective = "LL",
-                            Rcpp::Nullable<Rcpp::List> parameters = R_NilValue) {
+Rcpp::List new_NoiseKriging(std::string kernel) {
   NoiseKriging* ok = new NoiseKriging(kernel);
 
+  Rcpp::XPtr<NoiseKriging> impl_ptr(ok);
+
+  Rcpp::List obj;
+  obj.attr("object") = impl_ptr;
+  obj.attr("class") = "NoiseKriging";
+  return obj;
+}
+
+// [[Rcpp::export]]
+Rcpp::List new_NoiseKrigingFit(arma::vec y,
+                               arma::vec noise,
+                               arma::mat X,
+                               std::string kernel,
+                               std::string regmodel = "constant",
+                               bool normalize = false,
+                               std::string optim = "BFGS",
+                               std::string objective = "LL",
+                               Rcpp::Nullable<Rcpp::List> parameters = R_NilValue) {
+  Rcpp::List _parameters;
+  if (parameters.isNotNull()) {
+    Rcpp::List params(parameters);
+    _parameters = Rcpp::List::create();
+    if (params.containsElementNamed("sigma2")) {
+      _parameters.push_back(params["sigma2"], "sigma2");
+      _parameters.push_back(true, "has_sigma2");
+      _parameters.push_back(!(params.containsElementNamed("is_sigma2_estim") && !params["is_sigma2_estim"]),
+                            "is_sigma2_estim");
+    } else {
+      //_parameters.push_back(Rcpp::runif(1), "sigma2"); // turnaround mingw bug:
+      // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(true, "has_sigma2");
+      _parameters.push_back(-1, "sigma2");
+      _parameters.push_back(false, "has_sigma2");
+      _parameters.push_back(true, "is_sigma2_estim");
+    }
+    if (params.containsElementNamed("theta")) {
+      _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["theta"]), "theta");
+      _parameters.push_back(true, "has_theta");
+      _parameters.push_back(!(params.containsElementNamed("is_theta_estim") && !params["is_theta_estim"]),
+                            "is_theta_estim");
+    } else {
+      // Rcpp::NumericVector r = Rcpp::runif(X.n_cols); // turnaround mingw bug:
+      // https://github.com/msys2/MINGW-packages/issues/5019 _parameters.push_back(Rcpp::NumericMatrix(1, X.n_cols,
+      // r.begin()), "theta"); _parameters.push_back(true, "has_theta");
+      _parameters.push_back(Rcpp::NumericMatrix(0, 0), "theta");
+      _parameters.push_back(false, "has_theta");
+      _parameters.push_back(true, "is_theta_estim");
+    }
+    if (params.containsElementNamed("beta")) {
+      _parameters.push_back(Rcpp::as<Rcpp::NumericMatrix>(params["beta"]), "beta");
+      _parameters.push_back(true, "has_beta");
+      _parameters.push_back(!(params.containsElementNamed("is_beta_estim") && !params["is_beta_estim"]),
+                            "is_beta_estim");
+    } else {
+      _parameters.push_back(Rcpp::NumericVector(0), "beta");
+      _parameters.push_back(false, "has_beta");
+      _parameters.push_back(true, "is_beta_estim");
+    }
+  } else {
+    // Rcpp::NumericVector r = Rcpp::runif(X.n_cols); // turnaround mingw bug:
+    // https://github.com/msys2/MINGW-packages/issues/5019
+    _parameters = Rcpp::List::create(  // Rcpp::Named("sigma2") = Rcpp::runif(1),
+                                       // Rcpp::Named("has_sigma2") = true,
+        Rcpp::Named("sigma2") = -1,
+        Rcpp::Named("has_sigma2") = false,
+        Rcpp::Named("is_sigma2_estim") = true,
+        // Rcpp::Named("theta") = Rcpp::NumericMatrix(1, X.n_cols, r.begin()),
+        // Rcpp::Named("has_theta") = true,
+        Rcpp::Named("theta") = Rcpp::NumericMatrix(0, 0),
+        Rcpp::Named("has_theta") = false,
+        Rcpp::Named("is_theta_estim") = true,
+        Rcpp::Named("beta") = Rcpp::NumericVector(0),
+        Rcpp::Named("has_beta") = false,
+        Rcpp::Named("is_beta_estim") = true);
+  }
+
+  NoiseKriging* ok = new NoiseKriging(
+      std::move(y),
+      std::move(noise),
+      std::move(X),
+      kernel,
+      Trend::fromString(regmodel),
+      normalize,
+      optim,
+      objective,
+      NoiseKriging::Parameters{
+          (_parameters["has_sigma2"]) ? make_optional0<arma::vec>(_parameters["sigma2"]) : std::nullopt,
+          _parameters["is_sigma2_estim"],
+          (_parameters["has_theta"]) ? make_optional0<arma::mat>(_parameters["theta"]) : std::nullopt,
+          _parameters["is_theta_estim"],
+          (_parameters["has_beta"]) ? make_optional0<arma::colvec>(_parameters["beta"]) : std::nullopt,
+          _parameters["is_beta_estim"]});
+
+  Rcpp::XPtr<NoiseKriging> impl_ptr(ok);
+
+  Rcpp::List obj;
+  obj.attr("object") = impl_ptr;
+  obj.attr("class") = "NoiseKriging";
+  return obj;
+}
+
+// [[Rcpp::export]]
+void noisekriging_fit(Rcpp::List k,
+                      arma::vec y,
+                      arma::vec noise,
+                      arma::mat X,
+                      std::string regmodel = "constant",
+                      bool normalize = false,
+                      std::string optim = "BFGS",
+                      std::string objective = "LL",
+                      Rcpp::Nullable<Rcpp::List> parameters = R_NilValue) {
+  if (!k.inherits("NoiseKriging"))
+    Rcpp::stop("Input must be a NoiseKriging object.");
+  SEXP impl = k.attr("object");
+
+  Rcpp::XPtr<NoiseKriging> impl_ptr(impl);
+
   Rcpp::List _parameters;
   if (parameters.isNotNull()) {
     Rcpp::List params(parameters);
@@ -83,25 +191,32 @@ Rcpp::List new_NoiseKriging(arma::vec y,
         Rcpp::Named("is_beta_estim") = true);
   }
 
-  ok->fit(std::move(y),
-          std::move(noise),
-          std::move(X),
-          Trend::fromString(regmodel),
-          normalize,
-          optim,
-          objective,
-          NoiseKriging::Parameters{
-              (_parameters["has_sigma2"]) ? make_optional0<arma::vec>(_parameters["sigma2"]) : std::nullopt,
-              _parameters["is_sigma2_estim"],
-              (_parameters["has_theta"]) ? make_optional0<arma::mat>(_parameters["theta"]) : std::nullopt,
-              _parameters["is_theta_estim"],
-              (_parameters["has_beta"]) ? make_optional0<arma::colvec>(_parameters["beta"]) : std::nullopt,
-              _parameters["is_beta_estim"]});
+  impl_ptr->fit(std::move(y),
+                std::move(noise),
+                std::move(X),
+                Trend::fromString(regmodel),
+                normalize,
+                optim,
+                objective,
+                NoiseKriging::Parameters{
+                    (_parameters["has_sigma2"]) ? make_optional0<arma::vec>(_parameters["sigma2"]) : std::nullopt,
+                    _parameters["is_sigma2_estim"],
+                    (_parameters["has_theta"]) ? make_optional0<arma::mat>(_parameters["theta"]) : std::nullopt,
+                    _parameters["is_theta_estim"],
+                    (_parameters["has_beta"]) ? make_optional0<arma::colvec>(_parameters["beta"]) : std::nullopt,
+                    _parameters["is_beta_estim"]});
+}
 
-  Rcpp::XPtr<NoiseKriging> impl_ptr(ok);
+// [[Rcpp::export]]
+Rcpp::List noisekriging_copy(Rcpp::List k) {
+  if (!k.inherits("NoiseKriging"))
+    Rcpp::stop("Input must be a NoiseKriging object.");
+  SEXP impl = k.attr("object");
+  Rcpp::XPtr<NoiseKriging> impl_ptr(impl);
 
   Rcpp::List obj;
-  obj.attr("object") = impl_ptr;
+  Rcpp::XPtr<NoiseKriging> impl_copy(new NoiseKriging(*impl_ptr, ExplicitCopySpecifier{}));
+  obj.attr("object") = impl_copy;
   obj.attr("class") = "NoiseKriging";
   return obj;
 }
