@@ -12,5 +12,7 @@ BASEDIR=$(cd "$BASEDIR" && pwd -P)
 
 TMPDIR=$(mktemp -d)
 ARMA_LIBS=$(cmake -DLIBKRIGING_PATH="${LIBKRIGING_PATH}" -B "${TMPDIR}" "${BASEDIR}" | sed -n -r -e 's/^-- EXTERNAL_LIBS=(.*)$/\1/p')
+# avoid *-NOTFOUND to be displayed:
+ARMA_LIBS=$(echo "$ARMA_LIBS" | sed -e 's/[^ ]*-NOTFOUND[^ ]*//g')
 echo "${ARMA_LIBS}"
-rm -fr "${TMPDIR}"
\ No newline at end of file
+rm -fr "${TMPDIR}"
